package com.openeggbert.jdotnet.JDotNet.Properties;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;

/**
 * A concrete implementation of the IProperty interface, providing both
 * read and write access to a property of type T.
 * <p>
 * This class extends AbstractPropertyImpl, leveraging its protected "value" field
 * to store the property's data, and implements the getter and setter methods
 * as specified by the IProperty interface.
 *
 * @param <T> the type of the property's value
 */
public class ObservablePropertyImpl<T> extends PropertyImpl<T> implements IObservable<T>{

    private List<Consumer<T>> observers = new ArrayList<>();
    private final List<IPropertyChangeListener<T>> changeListeners = new ArrayList<>();
    private final List<IVetoablePropertyChangeListener<T>> vetoableChangeListeners = new ArrayList<>();

    public ObservablePropertyImpl(IReadOnlyProperty<T> readOnlyPropertyIn, IWriteOnlyProperty<T> writeOnlyPropertyIn) {
        super(readOnlyPropertyIn, writeOnlyPropertyIn);
    }

    public ObservablePropertyImpl(IReadOnlyProperty<T> readOnlyPropertyIn, IWriteOnlyProperty<T> writeOnlyPropertyIn, Validator<T> validator) {
        super(readOnlyPropertyIn, writeOnlyPropertyIn, validator);
    }
    public ObservablePropertyImpl(T initialValue) {
        super(initialValue);
    }
    public ObservablePropertyImpl(T initialValue, Validator<T> validator) {
        super(initialValue, validator);
    }
    @Override
    public void set(T valueIn) {
        T oldValue = this.get();

        boolean vetoed = false;
        if(!vetoableChangeListeners.isEmpty()) {
             vetoed = notifyVetoableChangeListeners(oldValue, valueIn);
        }
        if(vetoed) {
            //nothing to do
            return;
        }
        super.set(valueIn);
        if(!observers.isEmpty()) {
            notifyListeners(valueIn);
        }
        if(!changeListeners.isEmpty()) {
            notifyChangeListeners(oldValue, valueIn);
        }
    }

    private void notifyListeners(T value) {
        for (Consumer<T> listener : observers) {
            listener.accept(value);
        }
    }
    private void notifyChangeListeners(T oldValue, T newValue) {
        for (IPropertyChangeListener<T> listener : changeListeners) {
            listener.propertyChanged(oldValue, newValue);
        }
    }

    /**
     * Notifies all registered vetoable change listeners of a potential change
     * from the old value to the new value. If any listener vetoes the change by
     * throwing a {@code PropertyVetoException}, the method stops further notifications
     * and returns {@code true}.
     *
     * @param oldValue the previous value before the change
     * @param newValue the proposed new value
     * @return {@code true} if the change was vetoed by any listener; {@code false} otherwise
     */
    private boolean notifyVetoableChangeListeners(T oldValue, T newValue) {
        for (IVetoablePropertyChangeListener<T> listener : vetoableChangeListeners) {
            try {
                listener.vetoablePropertyChanged(oldValue, newValue);
            } catch (PropertyVetoException e) {
                return true;
            }
        }
        return false;
    }
    public void addChangeListener(IPropertyChangeListener<T> changeListener) {
        changeListeners.add(changeListener);
    }

    public void removeChangeListener(IPropertyChangeListener<T> changeListener) {
        changeListeners.remove(changeListener);
    }
    public void addVetoableChangeListener(IVetoablePropertyChangeListener<T> changeListener) {
        vetoableChangeListeners.add(changeListener);
    }

    public void removeVetoableChangeListener(IVetoablePropertyChangeListener<T> vetoablePropertyChangeListener) {
        vetoableChangeListeners.remove(vetoablePropertyChangeListener);
    }


    public void addObserver(Consumer<T> listener) {
        observers.add(listener);
    }

    public void removeObserver(Consumer<T> listener) {
        observers.remove(listener);
    }

    public boolean hasListeners() {
        return !observers.isEmpty() || !changeListeners.isEmpty() || !vetoableChangeListeners.isEmpty();
    }
    public boolean hasObservers() {
        return !observers.isEmpty();
    }
    public boolean hasChangeListeners() {
        return !changeListeners.isEmpty();
    }
    public boolean hasVetoableChangeListeners() {
        return !vetoableChangeListeners.isEmpty();
    }

}
