package com.openeggbert.jdotnet.JDotNet.Properties;

import java.util.Objects;

/**
 * A concrete implementation of the IProperty interface, providing both
 * read and write access to a property of type T.
 * <p>
 * This class extends AbstractPropertyImpl, leveraging its protected "value" field
 * to store the property's data, and implements the getter and setter methods
 * as specified by the IProperty interface.
 *
 * @param <T> the type of the property's value
 */
public class PropertyImpl<T> extends AbstractPropertyImpl<T> implements IProperty<T> {

    private IReadOnlyProperty<T> readOnlyProperty;
    private IWriteOnlyProperty<T> writeOnlyProperty;

    private final Validator<T> validator;

    public PropertyImpl(IReadOnlyProperty<T> readOnlyPropertyIn, IWriteOnlyProperty<T> writeOnlyPropertyIn) {
        this(readOnlyPropertyIn, writeOnlyPropertyIn, null);
    }
    public PropertyImpl(IReadOnlyProperty<T> readOnlyPropertyIn, IWriteOnlyProperty<T> writeOnlyPropertyIn, Validator<T> validator) {
        this.readOnlyProperty = Objects.requireNonNull(readOnlyPropertyIn, "readOnlyPropertyIn cannot be null.");
        this.writeOnlyProperty = Objects.requireNonNull(writeOnlyPropertyIn, "writeOnlyPropertyIn cannot be null.");
        this.value = null;
        this.validator = validator;
    }

    public PropertyImpl(T initialValue) {
        this(initialValue, null);
    }

    public PropertyImpl(T initialValue, Validator<T> validator) {
        this.value = initialValue;
        this.readOnlyProperty = null;
        this.writeOnlyProperty = null;
        this.validator = validator;
    }

    /**
     * Sets the value for the property.
     *
     * @param valueIn the value to set for the property
     */
    @Override
    public void set(T valueIn) {
        if (validator != null) {
            validator.validate(valueIn);
        }
        if (writeOnlyProperty != null) {
            writeOnlyProperty.set(valueIn);
        } else {
            this.value = valueIn;
        }
    }

    /**
     * Retrieves the current value of the property.
     *
     * @return the current value of the property
     */
    @Override
    public T get() {
        if (readOnlyProperty != null) {
            return readOnlyProperty.get();
        } else {
            return value;
        }
    }
}
