/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.openeggbert.jdotnet.System;

import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import lombok.Getter;

/**
 * The String_ class provides a set of utility functions to work with strings,
 * including checks for null or empty strings, formatting operations with templates,
 * and character repetitions. Helper methods are included to support locale-based
 * string formatting and basic type-dependent formatting specifications.
 *
 * This class is not instantiable directly, and its utility methods are designed
 * to mirror .NET-style string handling while integrating Java principles.
 */
public class String_ {

    protected String_() {
        //Not meant to be instantiated.
    }
    @Getter
    private String value;
    
    
    /**
     * Determines whether the specified string is either null or empty.
     *
     * @param string the string to check for null or emptiness.
     * @return true if the specified string is null or has a length of 0; false otherwise.
     */
    public static boolean IsNullOrEmpty(String string) {
        return string == null || string.isEmpty();
    }

    /**
     * Formats a template string by replacing placeholders in the form of `{index}` with corresponding
     * arguments from the provided list. The index must be a valid zero-based integer referencing
     * the `args` array.
     *
     * @param template the string template with placeholders in the format `{index}` to be replaced.
     *                 Must not be null.
     * @param args the array of objects to be used as replacements for the placeholders. Must not be null.
     * @return the formatted string with placeholders replaced by the corresponding arguments from `args`.
     * @throws IllegalArgumentException if `template` or `args` is null, if a placeholder is invalid,
     *                                  or if the placeholder index is out of range.
     */
    public static String Format(String template, Object... args) {
    if (template == null || args == null) {
        throw new IllegalArgumentException("Template and arguments must not be null.");
    }

    StringBuilder result = new StringBuilder();
    int length = template.length();

    for (int i = 0; i < length; i++) {
        char c = template.charAt(i);
        if (c == '{') {
            int closingBrace = template.indexOf('}', i);
            if (closingBrace == -1) {
                throw new IllegalArgumentException("Unmatched '{' in the template.");
            }

            String placeholder = template.substring(i + 1, closingBrace);
            int index;
            try {
                index = Integer.parseInt(placeholder);
            } catch (NumberFormatException e) {
                throw new IllegalArgumentException("Invalid placeholder: {" + placeholder + "}");
            }

            if (index < 0 || index >= args.length) {
                throw new IllegalArgumentException("Placeholder index out of range: " + index);
            }

            result.append(args[index].toString());
            i = closingBrace; // Skip to the closing brace
        } else {
            result.append(c);
        }
    }

    return result.toString();
    }
    
    /**
     * Constructs a new instance of the String_ class, initializing its value with a repeated
     * sequence of the specified character.
     *
     * @param ch the character to be repeated to form the string value.
     * @param times the number of times to repeat the character. Must be non-negative.
     * @throws Exception_ if the value of the parameter `times` is less than 0.
     */
    public String_(char ch, int times) {
        if(times < 0) {
            throw new Exception_("times cannot be less than 0");
        }
        if(times == 0) {value = "";return;}
        if(times == 1) {value = String.valueOf(ch); return;}
        StringBuilder sb = new StringBuilder();
        for(int i = 0;i< times;i++) {
            sb.append(ch);
        }
        value = sb.toString();
    }

    @Override
    public String toString() {
        return getValue();
    }
    
    public static String Format(Locale l, String format, Object... args) {
        if (format == null || args == null) {
            throw new IllegalArgumentException("Format string and arguments must not be null.");
        }

        StringBuilder result = new StringBuilder();
        int argIndex = 0;
        int length = format.length();

        for (int i = 0; i < length; i++) {
            char c = format.charAt(i);
            if (c == '%' && i + 1 < length) {
                char next = format.charAt(i + 1);
                if (next == '%') {
                    // Escaped percent sign
                    result.append('%');
                    i++;
                } else if (argIndex < args.length) {
                    result.append(formatArgument(next, args[argIndex++]));
                    i++;
                } else {
                    throw new IllegalArgumentException("Insufficient arguments provided for format string.");
                }
            } else {
                result.append(c);
            }
        }

        if (argIndex < args.length) {
            throw new IllegalArgumentException("Too many arguments provided for format string.");
        }

        return result.toString();
    }

    private static String formatArgument(char formatType, Object arg) {
        if (arg == null) {
            return "null";
        }

        switch (formatType) {
            case 'd': // Integer
                if (arg instanceof Number) {
                    return Integer.toString(((Number) arg).intValue());
                }
                break;
            case 'f': // Floating-point
                if (arg instanceof Number) {
                    return String.valueOf(((Number) arg).doubleValue());
                }
                break;
            case 's': // String
                return arg.toString();
            case 'x': // Hexadecimal
                if (arg instanceof Number) {
                    return Integer.toHexString(((Number) arg).intValue());
                }
                break;
            default:
                throw new IllegalArgumentException("Unsupported format type: %" + formatType);
        }

        throw new IllegalArgumentException("Invalid argument type for format type: %" + formatType);
    }

}
